/*
   AmziSub.C -- Extended Predicates for CGI Processing

   Link with AmziCGI.C

   Copyright (c) 1996-1998 Amzi! inc.  All Rights Reserved.
   See license.htm for Amzi! License Agreement
*/

/*#define	DEBUG*/

/* Solaris is Unix for CGI */
#ifdef __sun
#define  __unix__
#endif

#include <ctype.h>

#ifdef _WIN32
#include <direct.h>
#include <fcntl.h>
#include <io.h>
#include <windows.h>
#else
#define	_MAX_PATH	512
#endif

#include <stdio.h>
#include <string.h>
#include <time.h>

#ifdef __unix__
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif

#include "amzi.h"

#ifdef DEBUG
extern FILE *debugFile;
#endif

void doublechar1(char *s, char c);
void slashslash1(char *s);
void slashslash2(char* sout, const char* sin);


/* built-in predicates, callable from Prolog */
/* ----------------------------------------- */

/* function prototypes */

/* File/directory utilities */

TF EXPFUNC p_delfile(ENGid);
TF EXPFUNC p_chdir(ENGid);
TF EXPFUNC p_rmdir(ENGid);
TF EXPFUNC p_rename(ENGid);
#ifdef	_WIN32
TF EXPFUNC p_curdir(ENGid);
TF EXPFUNC p_mkdir(ENGid);
TF EXPFUNC p_setdrive(ENGid);
TF EXPFUNC p_getdrive(ENGid);
TF EXPFUNC p_findfirst(ENGid);
TF EXPFUNC p_findnext(ENGid);
#endif

/* SDK functions */

TF EXPFUNC p_gmtime_str(ENGid);
TF EXPFUNC p_gmtime_later_str(ENGid);

/* CGI support functions */

TF EXPFUNC p_cgiOpen(ENGid);
TF EXPFUNC p_cgiWrite(ENGid);
TF EXPFUNC p_cgiWriteBinaryFile(ENGid);
TF EXPFUNC p_cgiRewind(ENGid);
TF EXPFUNC p_cgiClose(ENGid);
TF EXPFUNC p_cgiExtractFileName(ENGid);
TF EXPFUNC p_cgiExtractPath(ENGid);
TF EXPFUNC p_cgiPathSeparator(ENGid);
TF EXPFUNC p_cgiDecodeURL(ENGid);

/* extended predicate table definitions */

PRED_INIT awinPreds[] = 
{
	/* File/directory utility predicates */
	{"delfile", 2, p_delfile},
	{"chdir", 2, p_chdir},
	{"rmdir", 2, p_rmdir},
	{"rename", 3, p_rename},
#ifdef	_WIN32
   {"curdir", 1, p_curdir},
	{"mkdir", 2, p_mkdir},
	{"setdrive", 1, p_setdrive},
	{"getdrive", 1, p_getdrive},
	{"findfirst", 3, p_findfirst},
	{"findnext", 1, p_findnext},
#endif

	/* SDK functions */
	{"gmtime_str", 1, p_gmtime_str},
	{"gmtime_later_str", 2, p_gmtime_later_str},

	/* CGI support functions */
	{"cgiOpen", 1, p_cgiOpen},
	{"cgiWrite", 1, p_cgiWrite},
	{"cgiWriteBinaryFile", 1, p_cgiWriteBinaryFile},
	{"cgiRewind", 0, p_cgiRewind},
	{"cgiClose", 0, p_cgiClose},
	{"cgiExtractFileName", 2, p_cgiExtractFileName},
	{"cgiExtractPath", 2, p_cgiExtractPath},
   {"cgiPathSeparator", 1, p_cgiPathSeparator},
   {"cgiDecodeURL", 2, p_cgiDecodeURL},

	{NULL, 0, NULL}
};


/* Global Variables */
/* ---------------- */

#define MAX_PROFILE 4096


/* Initialization Functions */
/* ------------------------ */

/* These are used when this is built as an LSX for debugging under the
   listener.  If you modify AMZICGI.PRO, we strongly suggest you build
   this file as an LSX, otherwise it is very hard to debug changes to
   the WebLS engine. */

/*
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpRes)
{
	switch(dwReason)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}

__declspec(dllexport) RC EXPFUNC InitPreds(ENGid eid, void* p)
*/
extern RC InitPreds(ENGid eid, void* p)
 {
	RC rc;
	char buf[80];

	rc = lsInitPreds(eid, awinPreds);
	if (rc)
	{
#ifdef	_WIN32
		wsprintf(buf, "Error #%d Loading CGI Predicates", rc);
		MessageBox(NULL, buf, "CGI LSX", MB_OK);
#else
		printf("Error #%d Loading CGI Predicates", rc);
#endif
	}

	return 0;
}


/* File and Directory Manipulation */
/* ------------------------------- */

/* chdir(Dir, Err) - Changes directory to Dir, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_chdir(ENGid CurEng)
{
	int	  rc;
	char  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	rc = chdir(Buffer);
	return(lsUnifyParm(CurEng, 2, cINT, &rc));
}

#ifdef _WIN32
/* curdir(Dir) - Returns the current working directory.
 */
TF EXPFUNC p_curdir(ENGid CurEng)
{
   char	buffer[_MAX_PATH], *p;
   int	zero = 0;

   p = getcwd(buffer, _MAX_PATH);
	if (p == NULL)
      return(FALSE);
   else
   {
      lsUnifyParm(CurEng, 1, cSTR, buffer);
      return(lsUnifyParm(CurEng, 2, cINT, &zero));
   }
}

/* mkdir(Dir, Err) - Makes the directory Dir, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_mkdir(ENGid CurEng)
{
	int	rc;
	char  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	rc = mkdir(Buffer);
	return(lsUnifyParm(CurEng, 2, cINT, &rc));
}
#endif /* _WIN32 */

/* rmdir(Dir, Err) - Removes directory Dir, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_rmdir(ENGid CurEng)
{
	int	 rc;
	char  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	rc = rmdir(Buffer);
	return(lsUnifyParm(CurEng, 2, cINT, &rc));
}

#ifdef	_WIN32
/* setdrive(Drive, Err) - Makes Drive current, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_setdrive(ENGid CurEng)
{
	char	  c;
	unsigned  i;
	char	  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	c = Buffer[0];
	if (c >= 'a' && c <= 'z')
		i = 1 + c - 'a';
	else if (c >= 'A' && c <= 'Z')
		i = 1 + c - 'A';
	else
		lsErrRaise(CurEng, "Bad drive spec sent to setdrive/1");

	_chdrive(i);
	return(TRUE);
}

/* getdrive(Drive) - Returns the current drive as a letter followed
   by a colon. */

TF EXPFUNC p_getdrive(ENGid CurEng)
{
	unsigned  i;
	char  drive[3];

	i = _getdrive();
	drive[0] = 'a' + i - 1;
	drive[1] = ':';
	drive[2] = '\0';
	lsUnifyParm(CurEng, 1, cSTR, drive);
	return(TRUE);
}

#endif /* _WIN32 */

/* delfile(File, Err) - Removes file File, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_delfile(ENGid eid)
{
	char	sFile[_MAX_PATH];
	RC		rc, ecode;

	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "input path too long");
	rc = lsGetParm(eid, 1, cSTR, sFile);

	if (remove(sFile))
		ecode = 0;
	else
		ecode = -1;

	rc = lsUnifyParm(eid, 2, cINT, &ecode);

	return TRUE;
}

/* rename(OldFile, NewFile, Err) - Renames file, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_rename(ENGid CurEng)
{
	RC	rc;
	int	ecode;
	char	sOldName[_MAX_PATH], sNewName[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	if (_MAX_PATH <= lsStrParmLen(CurEng, 2) )
		lsErrRaise(CurEng, "output path too long");
	rc = lsGetParm(CurEng, 1, cSTR, sOldName);
	rc = lsGetParm(CurEng, 2, cSTR, sNewName);

	ecode = rename(sOldName, sNewName);

	rc= lsUnifyParm(CurEng, 3, cINT, &ecode);
	return(TRUE);
}

#ifdef _WIN32

/* Global definitions for findfirst family */

long			FileHand;
struct			_finddata_t FT;
unsigned int	mask;

/* findfirst(Pattern, Mask, 
   fileinfo(Name,Attr,time(Hour,Min,Sec),date(Year,Mon,Day),Size))
   Finds the first file that matches the Pattern and Mask */

TF EXPFUNC p_findfirst(ENGid CurEng)
{
	unsigned short	attrib;
	long			time, date;
	unsigned short	hour, min, sec;
	unsigned short	year, month, day;
	TERM			fileinfoT, dateT, timeT;
	char			pattern[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, &pattern);
	lsGetParm(CurEng, 2, cINT, &mask);
	if (-1 == (FileHand = _findfirst(pattern, &FT)))
		return(FALSE);
	/* Make sure the one we have is an acceptable type */
	while (((FT.attrib & 31) & mask ) == 0 && (FT.attrib & 31) != mask)
	{
			if (-1 == _findnext(FileHand, &FT))
				return(FALSE);
	}

	lsMakeFA(CurEng, &fileinfoT, "fileinfo", 5);

	lsUnifyArg(CurEng, &fileinfoT, 1, cSTR, &(FT.name));

	attrib = FT.attrib;
	lsUnifyArg(CurEng, &fileinfoT, 2, cSHORT, &attrib);

	time = FT.time_access;
	lsMakeFA(CurEng, &timeT, "time", 3);
	hour = (time & 0xf800) >> 11;
	min = (time & 0x07e0) >> 5;
	sec = time & 0x001f;
	lsUnifyArg(CurEng, &timeT, 1, cSHORT, &hour);
	lsUnifyArg(CurEng, &timeT, 2, cSHORT, &min);
	lsUnifyArg(CurEng, &timeT, 3, cSHORT, &sec);
	lsUnifyArg(CurEng, &fileinfoT, 3, cTERM, &timeT);

	date = FT.time_access;
	lsMakeFA(CurEng, &dateT, "date", 3);
	year = (((date & 0xfe00) >> 9) + 80) % 100;
	month = (date & 0x1e0) >> 5;
	day = date & 0x001f;
	lsUnifyArg(CurEng, &dateT, 1, cSHORT, &year);
	lsUnifyArg(CurEng, &dateT, 2, cSHORT, &month);
	lsUnifyArg(CurEng, &dateT, 3, cSHORT, &day);
	lsUnifyArg(CurEng, &fileinfoT, 4, cTERM, &dateT);

	lsUnifyArg(CurEng, &fileinfoT, 5, cLONG, &(FT.size));

	return(lsUnifyParm(CurEng, 3, cTERM, &fileinfoT));
}

/* findnext(fileinfo(Name,Attr,time(Hour,Min,Sec),date(Year,Mon,Day),Size)) */
/*   Finds the next file that matches the previous Pattern and Mask */

TF EXPFUNC p_findnext(ENGid CurEng)
{
	TERM				fileinfoT, dateT, timeT;
	unsigned short	attrib;
	long				time, date;
	unsigned short	hour, min, sec;
	unsigned short	year, month, day;

	if (-1 == _findnext(FileHand, &FT))
		return(FALSE);

	/* Make sure the one we have is an acceptable type */
	while (((FT.attrib & 31) & mask) == 0 && (FT.attrib & 31) != mask)
	{
		if (-1 == _findnext(FileHand, &FT))
			return(FALSE);
	}

	lsMakeFA(CurEng, &fileinfoT, "fileinfo", 5);

	lsUnifyArg(CurEng, &fileinfoT, 1, cSTR, &(FT.name));

	attrib = FT.attrib;
	lsUnifyArg(CurEng, &fileinfoT, 2, cSHORT, &attrib);

	time = FT.time_access;
	lsMakeFA(CurEng, &timeT, "time", 3);
	hour = (time & 0xf800) >> 11;
	min = (time & 0x07e0) >> 5;
	sec = time & 0x001f;
	lsUnifyArg(CurEng, &timeT, 1, cSHORT, &hour);
	lsUnifyArg(CurEng, &timeT, 2, cSHORT, &min);
	lsUnifyArg(CurEng, &timeT, 3, cSHORT, &sec);
	lsUnifyArg(CurEng, &fileinfoT, 3, cTERM, &timeT);

	date = FT.time_access;
	lsMakeFA(CurEng, &dateT, "date", 3);
	year = (((date & 0xfe00) >> 9) + 80) % 100;
	month = (date & 0x1e0) >> 5;
	day = date & 0x001f;
	lsUnifyArg(CurEng, &dateT, 1, cSHORT, &year);
	lsUnifyArg(CurEng, &dateT, 2, cSHORT, &month);
	lsUnifyArg(CurEng, &dateT, 3, cSHORT, &day);
	lsUnifyArg(CurEng, &fileinfoT, 4, cTERM, &dateT);

	lsUnifyArg(CurEng, &fileinfoT, 5, cLONG, &(FT.size));

	return(lsUnifyParm(CurEng, 1, cTERM, &fileinfoT));
}

#endif /* _WIN32 */


/* SDK Functions */
/* ------------- */

/* gmtime_str(time) returns the current GMT time. */

TF EXPFUNC p_gmtime_str(ENGid eid)
{
	struct tm	*gmt;
	long		ltime;
	char		sBuf[40];
	TF			tf;

	time(&ltime);
	gmt = gmtime(&ltime);
	strftime(sBuf, 40, "%a, %d-%b-%Y %H:%M:%S GMT", gmt);
	tf = lsUnifyParm(eid, 1, cSTR, sBuf);

	return tf;
}

/* gmtime_later_str(increment, time) returns the current GMT time plus the increment specified. */

TF EXPFUNC p_gmtime_later_str(ENGid eid)
{
	struct tm	*gmt;
	long		ltime, inc;
	char		sBuf[40];
	TF			tf;
	RC			rc;

	rc = lsGetParm(eid, 1, cINT, &inc);
	if (rc != 0) return FALSE;
	time(&ltime);
	ltime = ltime + inc;
	gmt = gmtime(&ltime);
	strftime(sBuf, 40, "%a, %d-%b-%Y %H:%M:%S GMT", gmt);
	tf = lsUnifyParm(eid, 2, cSTR, sBuf);

	return tf;
}

/* CGI Support Functions */
/* --------------------- */

/* Global definitions for CGI output functions */

FILE	*outFile = NULL;
char	outName[_MAX_PATH];

/* These routines are used to open, write, rewind and close the
   output file, which is stdout in standard CGI */

/* cgiOpen(File) opens the named file (or if blank stdout) for write */

TF EXPFUNC p_cgiOpen(ENGid eid)
{
	int		rc;

	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "file name too long");
	rc = lsGetParm(eid, 1, cSTR, outName);

	if (strlen(outName) > 0)
	{
		outFile = fopen(outName, "w");
	}
	else
	{
		outFile = stdout;
	}

	if (outFile == NULL)
	{
#ifdef DEBUG
		fprintf(debugFile, "cgiOpen false");
		fflush(debugFile);
#endif
		return(FALSE);
	}
	else
	{
#ifdef DEBUG
		fprintf(debugFile, "cgiOpen true");
		fflush(debugFile);
#endif
		return(TRUE);
	}
}

/* cgiWrite(Term) writes the atom, string, structure, list to the output file */

TF EXPFUNC p_cgiWrite(ENGid eid)
{
	char	sBuf[4096];
	TERM	term;
	int		rc, ecode;

#ifdef DEBUG
	fprintf(debugFile, "cgiWrite");
	fflush(debugFile);
#endif

	rc = lsGetParm(eid, 1, cTERM, &term);
	if (rc != 0) return(FALSE);

	rc = lsTermToStr(eid, term, sBuf, 4096);
	if (rc != 0) return(FALSE);

	if (strlen(sBuf) == 0)
		return(TRUE);

#ifdef DEBUG
	fprintf(debugFile, "cgiWrite string=%s", sBuf);
	fflush(debugFile);
#endif

	ecode = fputs(sBuf, outFile);
	if (ecode < 0)
		return(FALSE);
	else
		return(TRUE);
}

/* cgiWriteBinaryFile(FileName) sends the file out */

TF EXPFUNC p_cgiWriteBinaryFile(ENGid eid)
{
	unsigned char	buf[8192];
	char	filename[_MAX_PATH];
	FILE	*f;
	int		rc, mode;
	size_t	count;

	rc = lsGetParm(eid, 1, cSTR, filename);
	if (rc != 0) return(FALSE);
	f = fopen(filename, "rb");
	if (f == NULL)
		return(FALSE);

	/* Change the output file to binary mode to prevent crlf translation */
#if _WIN32
	mode = _setmode(fileno(outFile), _O_BINARY);
	if (mode == -1) return(FALSE);
#endif
	while (!feof(f))
	{
		count = fread(buf, sizeof(unsigned char), 8192, f);
		if (ferror(f)) goto cgiWriteBinaryFileError;
		if (count > 0)
		{
			fwrite(buf, sizeof(unsigned char), count, outFile);
			if (ferror(outFile)) goto cgiWriteBinaryFileError;
		}
	}
	fclose(f);
	return(TRUE);

cgiWriteBinaryFileError:
#if _WIN32
	_setmode(fileno(outFile), mode);
#endif
	return(FALSE);
}

/* cgiRewind rewinds the output file to the beginning */

TF EXPFUNC p_cgiRewind(ENGid eid)
{
#ifdef DEBUG
	fprintf(debugFile, "cgiRewind");
	fflush(debugFile);
#endif
	if (outFile == stdout)
		rewind(outFile);
	else
	{
		if (outFile != NULL) fclose(outFile);
		outFile = fopen(outName, "w");
	}

	if (outFile == NULL)
		return(FALSE);
	else
		return(TRUE);
}


/* cgiClose closes the output file */

TF EXPFUNC p_cgiClose(ENGid eid)
{
#ifdef DEBUG
	fprintf(debugFile, "cgiClose");
	fflush(debugFile);
#endif

	if (outFile != stdout && outFile != NULL)
		fclose(outFile);
	return(TRUE);
}

/* cgiExtractProgramName gets the file name off the end of the path */

TF EXPFUNC p_cgiExtractFileName(ENGid eid)
{
	RC		rc;
	TF		tf;
	char	sPath[_MAX_PATH], sProg[_MAX_PATH], *p, *q;
	int		i;

#ifdef DEBUG
		fprintf(debugFile, "cgiExtractProgramName");
		fflush(debugFile);
#endif

	/* Get the path to purge files from */
	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "String too long");
	rc = lsGetParm(eid, 1, cSTR, sPath);
	p = strrchr(sPath, '\\');
	if (p == NULL)
	{
		p = strrchr(sPath, '/');
		if (p == NULL)
			p = &sPath[0];
	}
	p++;
	q = strchr(p, '.');
	if (q == NULL)
		q = &sPath[strlen(sPath)];
	i = 0;
	while (p != q)
	{
		sProg[i] = *p;
		p++; i++;
	}
	sProg[i] = '\0';

	tf = lsUnifyParm(eid, 2, cATOM, sProg);

	return tf;
}

/* cgiExtractPath removes the program name from a path */

TF EXPFUNC p_cgiExtractPath(ENGid eid)
{
	RC		rc;
	TF		tf;
	char	sPath[_MAX_PATH], *p;

#ifdef DEBUG
		fprintf(debugFile, "cgiExtractPath");
		fflush(debugFile);
#endif

	/* Get the path to purge files from */
	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "String too long");
	rc = lsGetParm(eid, 1, cSTR, sPath);
	p = strrchr(sPath, '\\');
	if (p == NULL)
	{
		p = strrchr(sPath, '/');
		if (p == NULL)
			p = &sPath[0];
	}
	p++;
   *p = '\0';

	tf = lsUnifyParm(eid, 2, cATOM, sPath);

	return tf;
}

/* cgiPathSeparator returns the path separator character as a string */

#if _WIN32
TF EXPFUNC p_cgiPathSeparator(ENGid eid)
{
	TF		tf;

   tf = lsUnifyParm(eid, 1, cSTR, "\\");
   return tf;
}
#endif

#ifdef __unix__
TF EXPFUNC p_cgiPathSeparator(ENGid eid)
{
	TF		tf;

   tf = lsUnifyParm(eid, 1, cSTR, "/");
   return tf;
}
#endif

/* Convert a URL by changing pluses to spaces and &nn to characters */

extern void decode_url(char *);

TF EXPFUNC p_cgiDecodeURL(ENGid eid)
{
   TF    tf;
   RC    rc;
   int   len;
   char  *buf;

   len = lsStrParmLen(eid, 1);
   buf = (char*)malloc(len);
   if (buf == NULL) return FALSE;
   rc = lsGetParm(eid, 1, cSTR, buf);
   if (rc != 0) return FALSE;
   decode_url(buf);
   tf = lsUnifyParm(eid, 2, cSTR, buf);
   return tf;
}


/* -------------------- Utility Routines -------------------- */

void doublechar1(char* s, char c)
{
	int	nslash=0;
	int	len=0;
	int	i;
	char* s1;
	char* s2;
	
	if (strlen(s) == 0) return;

	/* Count the quotes, then copy in place from right to
		left so we don't write on ourself. */
		
	s1 = s;
	while(*s1)
	{
		len++;
		if (*s1++ == c) nslash++;
	}
	
	s2 = s + len + nslash;
	s1 = s + len;
	
	for(i=0; i<=len; i++)
	{
		if (*s1 == c)
		{
			*s2-- = *s1;
			*s2-- = *s1--;
		}
		else
			*s2-- = *s1--;
	}
	
	return;
}

/* Any string of characters sent to Prolog is read using the
   Prolog reader, which interprets a backslash as an escape
   character.  So, this means if you really want a backslash,
   such as in a file path, then it has to be a double
   backslash.  This function makes that conversion. */

void slashslash2(char* sout, const char* sin)
{
	while(*sin)
	{
		if (*sin == '\\')
		{
			*sout++ = *sin;
			*sout++ = *sin++;
		}
		else
			*sout++ = *sin++;
	}
	*sout = *sin;
	return;
}

/* Same as slashslash2 except conversion is done in place.
   Therefore, the input buffer must be large enough to handle
   the expanded string. */

void slashslash1(char* s)
{
	int	nslash=0;
	int	len=0;
	int	i;
	char* s1;
	char* s2;
	
	if (strlen(s) == 0) return;

	/* Count the slashes, then copy in place from right to
		left so we don't write on ourself. */
		
	s1 = s;
	while(*s1)
	{
		len++;
		if (*s1++ == '\\') nslash++;
	}
	
	s2 = s + len + nslash;
	s1 = s + len;
	
	for(i=0; i<=len; i++)
	{
		if (*s1 == '\\')
		{
			*s2-- = *s1;
			*s2-- = *s1--;
		}
		else
			*s2-- = *s1--;
	}
	
	return;
}


